import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";

/* eslint-disable react/no-array-index-key */
import chunk from 'lodash/chunk';
import moment from 'moment';
import * as React from 'react';
import styled from 'styled-components';
import Text from '../../../primitives/Text';

var getMonthDatesGroupedByWeek = function getMonthDatesGroupedByWeek(year, month) {
  var monthMoment = moment().year(year).month(month);
  var start = monthMoment.clone().startOf('month').startOf('week');
  var end = monthMoment.clone().endOf('month').endOf('week');
  var arr = [];

  while (start.isSameOrBefore(end)) {
    arr.push(start.clone());
    start.add(1, 'day');
  }

  return chunk(arr, 7);
};

var getGroupedMonths = function getGroupedMonths(year) {
  var yearMoment = moment().year(year);
  var arr = Array.from({
    length: 12
  }).map(function (_unused, index) {
    return yearMoment.clone().month(index);
  });
  return chunk(arr, 4);
};

var getGroupedYears = function getGroupedYears(year) {
  var yearMoment = moment().year(year);
  var start = yearMoment.clone().subtract(6, 'years');
  var arr = [];

  while (arr.length !== 12) {
    arr.push(start.clone());
    start.add(1, 'year');
  }

  return chunk(arr, 4);
};

var Calendar = /*#__PURE__*/function (_React$Component) {
  _inherits(Calendar, _React$Component);

  var _super = _createSuper(Calendar);

  function Calendar(props) {
    var _this;

    _classCallCheck(this, Calendar);

    _this = _super.call(this, props);

    _this._changeGranularity = function () {
      _this.setState(function (prevState) {
        var newGranularity;

        switch (prevState.granularity) {
          case 'day':
            newGranularity = 'month';
            break;

          case 'month':
            newGranularity = 'year';
            break;

          default:
            break;
        }

        return {
          granularity: newGranularity
        };
      });
    };

    _this._getDisplayText = function (displayedMoment) {
      switch (_this.state.granularity) {
        case 'day':
          return displayedMoment.format('MMMM YYYY');

        case 'month':
          return displayedMoment.format('YYYY');

        default:
          return '';
      }
    };

    _this._navigate = function (dir, displayedMoment) {
      var fn = dir === 'forward' ? 'add' : 'subtract';
      var newDisplayedMoment = displayedMoment.clone();

      switch (_this.state.granularity) {
        case 'day':
          {
            // $FlowFixMe
            newDisplayedMoment[fn](1, 'month');
            break;
          }

        case 'month':
          {
            // $FlowFixMe
            newDisplayedMoment[fn](1, 'year');
            break;
          }

        case 'year':
          {
            // $FlowFixMe
            newDisplayedMoment[fn](12, 'years');
            break;
          }

        default:
          return;
      }

      _this.setState({
        displayedMonth: newDisplayedMoment.month(),
        displayedYear: newDisplayedMoment.year()
      });
    };

    _this._renderDayGrid = function () {
      return React.Children.toArray([_this._renderHeader(), _this._renderMonth(_this.state.displayedYear, _this.state.displayedMonth)]);
    };

    _this._renderGrid = function () {
      var content = null;

      switch (_this.state.granularity) {
        case 'day':
          content = _this._renderDayGrid();
          break;

        case 'month':
          content = _this._renderMonthGrid();
          break;

        case 'year':
          content = _this._renderYearGrid();
          break;

        default:
          break;
      }

      return /*#__PURE__*/React.createElement(Grid, null, content);
    };

    _this._renderHeader = function () {
      var now = moment();
      return /*#__PURE__*/React.createElement(HeaderContainer, null, [0, 1, 2, 3, 4, 5, 6].map(function (weekDayNum) {
        return /*#__PURE__*/React.createElement(HeaderCell, {
          key: weekDayNum
        }, /*#__PURE__*/React.createElement(GridText, null, now.day(weekDayNum).format('ddd.')));
      }));
    };

    _this._renderMonth = function (year, month) {
      var weeks = getMonthDatesGroupedByWeek(year, month);
      return /*#__PURE__*/React.createElement(MonthContainer, null, weeks.map(function (weekDays, weekNum) {
        return /*#__PURE__*/React.createElement(Row, {
          key: weekNum
        }, weekDays.map(function (day, dayNum) {
          var _this$props = _this.props,
              initialDate = _this$props.initialDate,
              onSelect = _this$props.onSelect;
          var isSelected = initialDate && day.isSame(initialDate, 'day');
          return /*#__PURE__*/React.createElement(Cell, {
            key: dayNum,
            onClick: function onClick() {
              return onSelect && onSelect(day);
            },
            $isSelected: isSelected,
            $isToday: day.isSame(moment(), 'day')
          }, /*#__PURE__*/React.createElement(GridText, {
            $isGray: day.month() !== month,
            $isSelected: isSelected
          }, day.date()));
        }));
      }));
    };

    _this._renderMonthGrid = function () {
      var handleClick = function handleClick(month) {
        _this.setState({
          displayedMonth: month.month(),
          displayedYear: month.year(),
          granularity: 'day'
        });
      };

      return getGroupedMonths(_this.state.displayedYear).map(function (months, groupNum) {
        return /*#__PURE__*/React.createElement(Row, {
          key: groupNum
        }, months.map(function (month, monthNum) {
          return /*#__PURE__*/React.createElement(Cell, {
            key: monthNum,
            onClick: function onClick() {
              return handleClick(month);
            }
          }, /*#__PURE__*/React.createElement(GridText, null, month.format('MMM.')));
        }));
      });
    };

    _this._renderNavigator = function () {
      var onSelect = _this.props.onSelect;
      var _this$state = _this.state,
          displayedMonth = _this$state.displayedMonth,
          displayedYear = _this$state.displayedYear,
          granularity = _this$state.granularity;
      var displayedMoment = moment().year(displayedYear).month(displayedMonth);
      return /*#__PURE__*/React.createElement(NavigatorContainer, null, /*#__PURE__*/React.createElement(DisplayContainer, {
        onClick: _this._changeGranularity
      }, (granularity === 'day' || granularity === 'month') && React.Children.toArray([/*#__PURE__*/React.createElement(DisplayText, null, _this._getDisplayText(displayedMoment)), /*#__PURE__*/React.createElement(Arrow, null)])), /*#__PURE__*/React.createElement(NavigatorButtonContainer, null, /*#__PURE__*/React.createElement(NavigatorButton, {
        onClick: function onClick() {
          return _this._navigate('backward', displayedMoment);
        }
      }, /*#__PURE__*/React.createElement(NavigatorButtonText, null, "<")), /*#__PURE__*/React.createElement(NavigatorButton, {
        onClick: function onClick() {
          return onSelect && onSelect(moment());
        }
      }, /*#__PURE__*/React.createElement(NavigatorButtonText, null, "Today")), /*#__PURE__*/React.createElement(NavigatorButton, {
        onClick: function onClick() {
          return _this._navigate('forward', displayedMoment);
        }
      }, /*#__PURE__*/React.createElement(NavigatorButtonText, null, ">"))));
    };

    _this._renderYearGrid = function () {
      var handleClick = function handleClick(year) {
        _this.setState({
          displayedYear: year.year(),
          granularity: 'month'
        });
      };

      return getGroupedYears(_this.state.displayedYear).map(function (years, groupNum) {
        return /*#__PURE__*/React.createElement(Row, {
          key: groupNum
        }, years.map(function (year, yearNum) {
          return /*#__PURE__*/React.createElement(Cell, {
            key: yearNum,
            onClick: function onClick() {
              return handleClick(year);
            }
          }, /*#__PURE__*/React.createElement(GridText, null, year.format('YYYY')));
        }));
      });
    };

    var _initialDate = _this.props.initialDate;

    if (!_initialDate || !_initialDate.isValid()) {
      _initialDate = moment();
    }

    _this.state = {
      displayedYear: _initialDate.year(),
      displayedMonth: _initialDate.month(),
      granularity: 'day'
    };
    return _this;
  }

  _createClass(Calendar, [{
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement(CalendarContainer, {
        $alignRight: this.props.alignRight || false
      }, this._renderNavigator(), this._renderGrid());
    }
  }]);

  return Calendar;
}(React.Component);

export default Calendar;
var Arrow = styled.span.withConfig({
  displayName: "Calendar__Arrow",
  componentId: "g59w5t-0"
})(["border-color:#999 transparent transparent;border-style:solid;border-width:5px 5px 2.5px;height:0;margin-left:3px;width:0;position:relative;"]);
var CalendarContainer = styled.div.attrs(function (props) {
  return {
    'data-component': 'calendar'
  };
}).withConfig({
  displayName: "Calendar__CalendarContainer",
  componentId: "g59w5t-1"
})(["", ""], function (_ref) {
  var $alignRight = _ref.$alignRight;
  return $alignRight && "\n    position: absolute;\n    border: 1px solid #ccc;\n    right: ".concat($alignRight, "px;\n    background-color: #fff;\n    ");
});
var Cell = styled.div.withConfig({
  displayName: "Calendar__Cell",
  componentId: "g59w5t-2"
})(["align-items:center;cursor:pointer;display:flex;flex:1;height:25px;justify-content:center;", " ", ""], function (_ref2) {
  var $isToday = _ref2.$isToday;
  return $isToday && "\n    border: 1px solid #ccc;\n    height: 23px;\n  ";
}, function (_ref3) {
  var $isSelected = _ref3.$isSelected;
  return $isSelected ? "\n    background-color: #445;\n  " : "\n    &:hover {\n      background-color: #eee;\n    }\n  ";
});
var DisplayContainer = styled.div.withConfig({
  displayName: "Calendar__DisplayContainer",
  componentId: "g59w5t-3"
})(["align-items:center;cursor:pointer;display:flex;padding:8px;&:hover{background-color:#eee;", "{border-top-color:#666;}}"], Arrow);
var DisplayText = styled(Text).withConfig({
  displayName: "Calendar__DisplayText",
  componentId: "g59w5t-4"
})(["font-size:14px;"]);
var Grid = styled.div.withConfig({
  displayName: "Calendar__Grid",
  componentId: "g59w5t-5"
})(["width:280px;"]);
var GridText = styled(Text).withConfig({
  displayName: "Calendar__GridText",
  componentId: "g59w5t-6"
})(["font-size:12.6px;", ";", ";"], function (_ref4) {
  var $isGray = _ref4.$isGray;
  return $isGray && "color: #c3c3c3;";
}, function (_ref5) {
  var $isSelected = _ref5.$isSelected;
  return $isSelected && "color: white;";
});
var HeaderCell = styled.div.withConfig({
  displayName: "Calendar__HeaderCell",
  componentId: "g59w5t-7"
})(["align-items:center;cursor:default;display:flex;flex:1;height:25px;justify-content:center;"]);
var MonthContainer = styled.div.withConfig({
  displayName: "Calendar__MonthContainer",
  componentId: "g59w5t-8"
})(["display:flex;flex-direction:column;"]);
var NavigatorButton = styled.button.attrs(function (props) {
  return {
    tabIndex: -1,
    type: 'button'
  };
}).withConfig({
  displayName: "Calendar__NavigatorButton",
  componentId: "g59w5t-9"
})(["background:none;border:1px solid #ccc;cursor:pointer;&:hover{background-color:#eee;}&:not(:last-child){margin-right:7px;}"]);
var NavigatorButtonContainer = styled.div.withConfig({
  displayName: "Calendar__NavigatorButtonContainer",
  componentId: "g59w5t-10"
})(["display:flex;padding:3px;"]);
var NavigatorButtonText = styled(Text).withConfig({
  displayName: "Calendar__NavigatorButtonText",
  componentId: "g59w5t-11"
})([""]);
var NavigatorContainer = styled.div.withConfig({
  displayName: "Calendar__NavigatorContainer",
  componentId: "g59w5t-12"
})(["display:flex;justify-content:space-between;padding:7px;"]);
var Row = styled.div.withConfig({
  displayName: "Calendar__Row",
  componentId: "g59w5t-13"
})(["display:flex;"]);
var HeaderContainer = styled(Row).withConfig({
  displayName: "Calendar__HeaderContainer",
  componentId: "g59w5t-14"
})(["border-bottom:1px solid #ccc;display:flex;"]);