import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import * as React from 'react';
import Icon from '../Icon'; // Note that this is not the Modal we export. It has a slide animation on native.

import Modal from './Modal';
import { ActionRow, BackLink, Label, WebContainer } from './StyledComponents.web';
export default function Dropdown(_ref) {
  var actions = _ref.actions,
      _ref$icon = _ref.icon,
      icon = _ref$icon === void 0 ? 'more-vert' : _ref$icon,
      align = _ref.align,
      _ref$width = _ref.width,
      width = _ref$width === void 0 ? 300 : _ref$width,
      children = _ref.children,
      _ref$iconColor = _ref.iconColor,
      iconColor = _ref$iconColor === void 0 ? 'black' : _ref$iconColor,
      testID = _ref.testID;

  var _React$useState = React.useState(false),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      active = _React$useState2[0],
      setActive = _React$useState2[1];

  var _React$useState3 = React.useState({}),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      modalPlacement = _React$useState4[0],
      setModalPlacement = _React$useState4[1];

  var containerRef = React.useRef(null);

  var updatePlacement = function updatePlacement() {
    if (containerRef && containerRef.current) {
      var dims = containerRef.current.getBoundingClientRect();
      var left = Math.max(dims.left, 5);
      var right = Math.max(window.innerWidth - (dims.left + dims.width), 5);

      var _align = align || (left < right * 2 ? 'left' : 'right');

      if (_align === 'left') {
        right = null;

        if (left + width > window.innerWidth) {
          // the left position cannot be too high, it should not push the dropdown over the edge of the screen
          left = window.innerWidth - (width + 5);
        }
      } else {
        left = null;

        if (right + width > window.innerWidth) {
          // the right position cannot be too high, it should not push the dropdown over the edge of the screen
          right = window.innerWidth - (width + 5);
        }
      }

      var top = dims.bottom + 3;
      var height = actions.length * 40;

      if (top + height + 10 > window.innerHeight) {
        top = window.innerHeight - height - 10;
      }

      setModalPlacement({
        type: 'dropdown',
        top: top,
        height: height,
        width: width,
        align: _align,
        left: left,
        right: right
      });
    }
  };

  React.useEffect(function () {
    updatePlacement();
  }, [active]);

  var close = function close() {
    return setActive(false);
  };

  var renderAction = function renderAction(action) {
    return /*#__PURE__*/React.createElement(ActionRow, {
      key: action.label,
      onPress: function onPress() {
        close();
        action.onPress();
      }
    }, action.icon && /*#__PURE__*/React.createElement(Icon, {
      name: action.icon
    }), /*#__PURE__*/React.createElement(Label, null, action.label));
  };

  return /*#__PURE__*/React.createElement(WebContainer, {
    "data-testid": testID
  }, /*#__PURE__*/React.createElement("div", {
    style: {
      display: 'inline-block',
      cursor: 'pointer',
      padding: '0px 5px'
    },
    onClick: function onClick() {
      return setActive(true);
    },
    ref: containerRef
  }, children || /*#__PURE__*/React.createElement(Icon, {
    name: icon,
    color: iconColor
  })), active ? /*#__PURE__*/React.createElement(Modal, {
    onRequestClose: close,
    placement: modalPlacement,
    visible: true,
    hideHeader: true
  }, /*#__PURE__*/React.createElement(BackButton, {
    onPress: close
  }), actions.map(renderAction)) : null);
}

var BackButton = function BackButton(_ref2) {
  var onPress = _ref2.onPress;
  return /*#__PURE__*/React.createElement(BackLink, {
    onPress: onPress
  }, /*#__PURE__*/React.createElement(Icon, {
    name: "arrow-back-ios"
  }));
};