import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import * as React from 'react';
import Colors from '../../apis/Colors';
import { useDeviceKind } from '../../apis/DeviceKind';
import { Popover } from '../../internals/Popover.web';
import Icon from '../Icon';
import { Context, useMenuContext } from './MenuContext';
import { ActionRow, MenuItemIcon, MenuItemLabel, PopoverContentContainer, TriggerContainer } from './StyledComponents.web';
export default function Menu(_ref) {
  var align = _ref.align,
      children = _ref.children,
      className = _ref.className,
      _ref$icon = _ref.icon,
      icon = _ref$icon === void 0 ? 'menu' : _ref$icon,
      _ref$iconColor = _ref.iconColor,
      iconColor = _ref$iconColor === void 0 ? Colors.BLACK : _ref$iconColor,
      _ref$maxHeight = _ref.maxHeight,
      maxHeight = _ref$maxHeight === void 0 ? 150 : _ref$maxHeight,
      maxWidth = _ref.maxWidth,
      _ref$minWidth = _ref.minWidth,
      minWidth = _ref$minWidth === void 0 ? 120 : _ref$minWidth,
      testID = _ref.testID,
      trigger = _ref.trigger,
      triggerStyle = _ref.triggerStyle;

  var _React$useState = React.useState(false),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      active = _React$useState2[0],
      setActive = _React$useState2[1];

  var containerRef = React.useRef(null);
  var deviceKind = useDeviceKind();

  var close = function close() {
    return setActive(false);
  };

  var getPopoverPlacement = function getPopoverPlacement(anchorRect, popupContentRect) {
    var right = anchorRect.left - popupContentRect.width + anchorRect.width;
    var distanceFromRight = window.innerWidth - anchorRect.left - anchorRect.width;

    var _align = align || (anchorRect.left < distanceFromRight * 2 ? 'left' : 'right');

    return {
      top: anchorRect.bottom,
      left: _align === 'left' ? anchorRect.left : right
    };
  };

  return /*#__PURE__*/React.createElement(Context.Provider, {
    value: {
      close: close,
      active: active
    }
  }, /*#__PURE__*/React.createElement(TriggerContainer, {
    ref: containerRef,
    className: className,
    onClick: function onClick() {
      return setActive(true);
    },
    testID: testID,
    style: triggerStyle
  }, typeof trigger === 'function' ? trigger(active) : trigger || /*#__PURE__*/React.createElement(Icon, {
    name: icon,
    color: iconColor
  })), active ? /*#__PURE__*/React.createElement(Popover, {
    anchorEl: containerRef.current,
    onRequestClose: function onRequestClose() {
      return setActive(false);
    },
    placement: getPopoverPlacement,
    visible: true
  }, /*#__PURE__*/React.createElement(PopoverContentContainer, {
    $minWidth: minWidth,
    $maxWidth: maxWidth,
    $maxHeight: maxHeight,
    $deviceKind: deviceKind
  }, children)) : null);
}

var Item = function Item(_ref2) {
  var children = _ref2.children,
      icon = _ref2.icon,
      numberOfLines = _ref2.numberOfLines,
      _onPress = _ref2.onPress;
  var context = useMenuContext();
  return /*#__PURE__*/React.createElement(ActionRow, {
    onPress: function onPress() {
      context.close();
      _onPress === null || _onPress === void 0 ? void 0 : _onPress();
    }
  }, typeof children === 'string' ? /*#__PURE__*/React.createElement(React.Fragment, null, icon ? /*#__PURE__*/React.createElement(MenuItemIcon, {
    name: icon
  }) : null, /*#__PURE__*/React.createElement(MenuItemLabel, {
    numberOfLines: numberOfLines,
    $hasIcon: !!icon
  }, children)) : children);
};

Menu.Item = Item;