import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React from 'react';
import styled from 'styled-components';
import { useDeviceKind } from '../../../apis/DeviceKind';
import IconButton from '../../../components/IconButton';
import FlexDiv from '../../../internals/web/FlexDiv';
import { DivViewBase } from '../../../internals/web/ViewBase';
import Overlay from '../../../primitives/Overlay';
import Text from '../../../primitives/Text';
import View from '../../../primitives/View';

function Modal(_ref) {
  var children = _ref.children,
      _ref$hideHeader = _ref.hideHeader,
      hideHeader = _ref$hideHeader === void 0 ? false : _ref$hideHeader,
      onRequestClose = _ref.onRequestClose,
      visible = _ref.visible,
      placement = _ref.placement,
      title = _ref.title,
      renderFooter = _ref.renderFooter;
  var deviceKind = useDeviceKind();

  var _React$useState = React.useState(null),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      modalContentLayout = _React$useState2[0],
      setModalContentLayout = _React$useState2[1];

  var _React$useState3 = React.useState(false),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      mouseEntered = _React$useState4[0],
      setMouseEntered = _React$useState4[1];

  var _React$useState5 = React.useState(false),
      _React$useState6 = _slicedToArray(_React$useState5, 2),
      mouseDown = _React$useState6[0],
      setMouseDown = _React$useState6[1];

  var _React$useState7 = React.useState(false),
      _React$useState8 = _slicedToArray(_React$useState7, 2),
      cancelClose = _React$useState8[0],
      setCancelClose = _React$useState8[1];

  var modalContentContainerRef = React.useRef(null);
  var prevVisibleRef = React.useRef(visible);
  React.useEffect(function () {
    if (visible && !prevVisibleRef.current) {
      var domEl = modalContentContainerRef.current;

      if (domEl && document.activeElement !== domEl && !domEl.contains(document.activeElement)) {
        // only focus if modal container or its children does not have focus
        //@ts-ignore
        document.activeElement.blur();
      }
    }

    prevVisibleRef.current = visible;
  }, [visible]);

  var onModalContentKeyDown = function onModalContentKeyDown(event) {
    if (event.keyCode === 27) {
      // escape
      event.stopPropagation();
      onRequestClose();
    }
  };

  var onModalContentLayout = function onModalContentLayout(event) {
    setModalContentLayout(event.nativeEvent.layout);
  };

  return /*#__PURE__*/React.createElement(Overlay, {
    onPress: function onPress() {
      if (!cancelClose) onRequestClose();
      setCancelClose(false);
    },
    visible: visible
  }, /*#__PURE__*/React.createElement(ModalContentContainer, {
    $deviceKind: deviceKind,
    ref: modalContentContainerRef,
    $layout: modalContentLayout,
    onKeyDown: onModalContentKeyDown,
    onLayout: onModalContentLayout,
    $placement: placement,
    onMouseEnter: function onMouseEnter(event) {
      setMouseEntered(true);
    },
    onMouseLeave: function onMouseLeave(event) {
      setMouseEntered(false);
      setCancelClose(mouseDown);
    },
    onMouseUp: function onMouseUp(event) {
      setMouseDown(false);
      setCancelClose(mouseEntered);
    },
    onMouseDown: function onMouseDown(event) {
      setMouseDown(true);
    }
  }, title || !hideHeader ? /*#__PURE__*/React.createElement(ModalHeader, null, /*#__PURE__*/React.createElement(ModalTitleContainer, null, /*#__PURE__*/React.createElement(ModalTitleText, {
    $deviceKind: deviceKind
  }, title)), /*#__PURE__*/React.createElement(ModalCloseIcon, {
    name: "clear",
    onPress: onRequestClose
  })) : null, children, typeof renderFooter === 'function' ? /*#__PURE__*/React.createElement(ModalFooter, null, renderFooter()) : null));
}

export default Modal;
export var ModalContentContainer = styled(DivViewBase).withConfig({
  displayName: "Modal__ModalContentContainer",
  componentId: "sc-1ms5etl-0"
})(["background-color:white;position:absolute;border-radius:4px;overflow:hidden;&:focus{outline:none;}", " ", ";"], function (props) {
  return props.$deviceKind === 'phone' || props.$deviceKind === 'large-phone' ? "\n      left: 0;\n      right: 0;\n      top: 0;\n      bottom: 0;\n      border-radius: 0px;\n      " : desktopStyle(props);
}, function (props) {
  return props.$deviceKind === 'desktop' || props.$deviceKind === 'tablet' ? "\n      @media (max-height: 768px) {\n        max-height: 100%;\n      }\n      " : '';
});
var ModalHeader = styled(View).withConfig({
  displayName: "Modal__ModalHeader",
  componentId: "sc-1ms5etl-1"
})(["flex:none;flex-direction:row;justify-content:space-between;background-color:white;border-bottom:1px solid #dddbda;height:55px;"]);
export var ModalTitleContainer = styled(View).withConfig({
  displayName: "Modal__ModalTitleContainer",
  componentId: "sc-1ms5etl-2"
})(["align-items:center;justify-content:center;position:absolute;top:0;bottom:0;left:70px;right:70px;"]);
var ModalTitleText = styled(Text).attrs(function (props) {
  return {
    numberOfLines: 2
  };
}).withConfig({
  displayName: "Modal__ModalTitleText",
  componentId: "sc-1ms5etl-3"
})(["font-size:20px;color:#2b2826;line-height:20px;", ""], function (props) {
  return props.$deviceKind === 'phone' || props.$deviceKind === 'large-phone' ? "\n      font-size: 16px;\n      color: #2b2826;\n      line-height: 16px;\n      text-align: center;\n      " : '';
});
var ModalCloseIcon = styled(IconButton).withConfig({
  displayName: "Modal__ModalCloseIcon",
  componentId: "sc-1ms5etl-4"
})(["position:absolute;right:20px;top:15px;"]);
var ModalFooter = styled(FlexDiv).withConfig({
  displayName: "Modal__ModalFooter",
  componentId: "sc-1ms5etl-5"
})(["flex:none;flex-direction:row;justify-content:space-between;background-color:#f1f1f0;border-top:1px solid #dddbda;height:64px;padding:16px;margin-top:auto;"]);

var desktopStyle = function desktopStyle(_ref2) {
  var $layout = _ref2.$layout,
      $placement = _ref2.$placement;

  if ($placement && $placement.type === 'dropdown' && $placement.align === 'left') {
    return "\n      border-radius: 2px;\n      left: ".concat($placement.left, "px;\n      top: ").concat($placement.top, "px;\n      height: ").concat($placement.height, "px;\n      width: ").concat($placement.width, "px;        \n    ");
  }

  if ($placement && $placement.type === 'dropdown' && $placement.align === 'right') {
    return "\n      border-radius: 2px;\n      right: ".concat($placement.right, "px;\n      top: ").concat($placement.top, "px;\n      height: ").concat($placement.height, "px;\n      width: ").concat($placement.width, "px;        \n    ");
  }

  if ($placement && $placement.type === 'fullscreen') {
    return "\n      left: 0;\n      right: 0;\n      top: 0;\n      bottom: 0;\n      border-radius: 0px;\n    ";
  }

  return "\n    left: 0;\n    right: 0;\n    margin: auto;\n    max-height: 768px;\n    min-height: 200px;\n    width: 576px;\n    ".concat($layout ? "\n          top: calc(50vh - ".concat(Math.ceil($layout.height / 2), "px);\n        ") : "\n          opacity: 0;\n        ", ";\n    ");
};