import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React from 'react';
import ReactSelect, { createFilter } from 'react-select';
import FontContext from '../../../primitives/Font/FontContext';
import { ClearIndicator } from './ClearIndicator';
import { DropdownIndicator } from './DropdownIndicator';
import { MenuList } from './MenuList';
var Select = React.forwardRef(function Select(_ref, _ref3) {
  var _ref2, _options$find;

  var accessibilityLabel = _ref.accessibilityLabel,
      autoFocus = _ref.autoFocus,
      className = _ref.className,
      clearable = _ref.clearable,
      filterOption = _ref.filterOption,
      getDropdownItemLayout = _ref.getDropdownItemLayout,
      isLoading = _ref.isLoading,
      multi = _ref.multi,
      noResultsText = _ref.noResultsText,
      onBlur = _ref.onBlur,
      onDropdownEndReached = _ref.onDropdownEndReached,
      onDropdownEndReachedThreshold = _ref.onDropdownEndReachedThreshold,
      onFocus = _ref.onFocus,
      onInputChange = _ref.onInputChange,
      onValueChange = _ref.onValueChange,
      options = _ref.options,
      pageSize = _ref.pageSize,
      placeholder = _ref.placeholder,
      readOnly = _ref.readOnly,
      renderDropdownIcon = _ref.renderDropdownIcon,
      renderOption = _ref.renderOption,
      searchable = _ref.searchable,
      style = _ref.style,
      testID = _ref.testID,
      value = _ref.value;

  var _React$useContext = React.useContext(FontContext),
      fontFamily = _React$useContext.fontFamily,
      fontSize = _React$useContext.fontSize; // control input value as we need it to filter options in the custom MenuList


  var _React$useState = React.useState(''),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      inputValue = _React$useState2[0],
      setInputValue = _React$useState2[1];

  var handleInputChange = function handleInputChange(newValue, actionMeta) {
    setInputValue(newValue);
    onInputChange === null || onInputChange === void 0 ? void 0 : onInputChange(newValue, actionMeta.action === 'input-change');
  };

  var selectRef = React.useRef(null);
  var selectedOption = multi ? ((_ref2 = value) !== null && _ref2 !== void 0 ? _ref2 : []).map(function (val) {
    return options === null || options === void 0 ? void 0 : options.find(function (op) {
      return op.value === val;
    });
  }).filter(Boolean) : (_options$find = options === null || options === void 0 ? void 0 : options.find(function (op) {
    return op.value === value;
  })) !== null && _options$find !== void 0 ? _options$find : null;
  return /*#__PURE__*/React.createElement(ReactSelect, {
    ref: function ref(select) {
      selectRef.current = select;

      if (_ref3) {
        if (typeof _ref3 === 'function') {
          _ref3(select);
        } else {
          _ref3.current = select;
        }
      }
    },
    "aria-label": accessibilityLabel,
    autoFocus: autoFocus,
    className: className,
    components: {
      ClearIndicator: ClearIndicator,
      DropdownIndicator: DropdownIndicator,
      MenuList: MenuList
    },
    filterOption: filterOption !== null && filterOption !== void 0 ? filterOption : createFilter({}),
    getDropdownItemLayout: getDropdownItemLayout,
    inputValue: inputValue,
    isClearable: clearable !== null && clearable !== void 0 ? clearable : true,
    isDisabled: readOnly,
    isLoading: isLoading,
    isMulti: multi // Always set `isSearchable` to false so that ReactSelect renders a fake
    // input. We pass a custom prop `searchable` down instead. If searchable,
    // we render the input inside the menu.
    ,
    isSearchable: false,
    searchable: searchable,
    menuPortalTarget: document.body,
    noOptionsMessage: noResultsText ? function () {
      return noResultsText;
    } : undefined,
    onBlur: onBlur !== null && onBlur !== void 0 ? onBlur : undefined,
    onChange: function onChange(option) {
      if (multi) {
        onValueChange === null || onValueChange === void 0 ? void 0 : onValueChange(option ? option.map(function (op) {
          return op.value;
        }) : []);
      } else {
        var _option$value;

        onValueChange === null || onValueChange === void 0 ? void 0 : onValueChange((_option$value = option === null || option === void 0 ? void 0 : option.value) !== null && _option$value !== void 0 ? _option$value : null);
      }
    },
    onDropdownEndReached: onDropdownEndReached,
    onDropdownEndReachedThreshold: onDropdownEndReachedThreshold,
    onFocus: onFocus !== null && onFocus !== void 0 ? onFocus : undefined,
    onInputChange: handleInputChange,
    openMenuOnFocus: true,
    options: options,
    pageSize: pageSize,
    placeholder: placeholder !== null && placeholder !== void 0 ? placeholder : '',
    renderDropdownIcon: renderDropdownIcon,
    renderOption: renderOption,
    selectRef: selectRef,
    styles: _objectSpread(_objectSpread({}, styles), {}, {
      container: function container() {
        return _objectSpread({
          fontFamily: fontFamily,
          fontSize: fontSize
        }, style);
      }
    }),
    testID: testID,
    value: selectedOption
  });
});
export default Select;
var styles = {
  clearIndicator: function clearIndicator(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      padding: 0
    });
  },
  control: function control(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      borderWidth: 0,
      boxShadow: 'none',
      minHeight: 0
    });
  },
  dropdownIndicator: function dropdownIndicator(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      padding: 0
    });
  },
  indicatorsContainer: function indicatorsContainer(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      paddingLeft: 0,
      paddingRight: 0
    });
  },
  indicatorSeparator: function indicatorSeparator(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      backgroundColor: 'transparent'
    });
  },
  input: function input(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      visibility: 'hidden'
    });
  },
  noOptionsMessage: function noOptionsMessage(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      color: '#999'
    });
  },
  placeholder: function placeholder(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      color: '#aaa'
    });
  },
  valueContainer: function valueContainer(provided) {
    return _objectSpread(_objectSpread({}, provided), {}, {
      paddingLeft: 0,
      paddingRight: 0
    });
  }
};