import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
import moment from 'moment';
import * as React from 'react';
import { DatePickerIOS } from 'react-native';
import PickerNative from '../../../internals/Picker.native';
import { DEFAULT_TIME_DISPLAY_FORMAT, DEFAULT_TIME_VALUE_FORMAT } from '../constants';
import { PlaceholderText, ValueText } from '../StyledComponents';

var getDefaultTime = function getDefaultTime() {
  return moment().startOf('minute').toDate();
};

var TimePicker = /*#__PURE__*/function (_React$Component) {
  _inherits(TimePicker, _React$Component);

  var _super = _createSuper(TimePicker);

  _createClass(TimePicker, [{
    key: "blur",
    value: function blur() {
      this._picker && this._picker.blur();
    }
  }, {
    key: "focus",
    value: function focus() {
      this._picker && this._picker.focus();
    }
  }]);

  function TimePicker(props) {
    var _this;

    _classCallCheck(this, TimePicker);

    _this = _super.call(this, props);

    _this._capturePickerRef = function (ref) {
      _this._picker = ref;
    };

    _this._onTimeChange = function (time) {
      _this.setState({
        selectedTime: time
      });
    };

    _this._renderPopup = function () {
      return /*#__PURE__*/React.createElement(DatePickerIOS, {
        date: _this.state.selectedTime || getDefaultTime(),
        mode: "time",
        onDateChange: _this._onTimeChange
      });
    };

    _this._renderValue = function () {
      var _this$props = _this.props,
          displayFormat = _this$props.displayFormat,
          placeholder = _this$props.placeholder,
          value = _this$props.value,
          valueFormat = _this$props.valueFormat;
      var time = moment(value || '', valueFormat);
      var timeStr = time.isValid() ? time.format(displayFormat) : '';

      if (!timeStr) {
        return /*#__PURE__*/React.createElement(PlaceholderText, null, placeholder);
      }

      return /*#__PURE__*/React.createElement(ValueText, null, timeStr);
    };

    _this._selectTime = function (time) {
      var _this$props2 = _this.props,
          onValueChange = _this$props2.onValueChange,
          valueFormat = _this$props2.valueFormat;

      _this.blur();

      _this.setState({
        selectedTime: time
      });

      if (onValueChange) {
        var timeMoment = moment(time);

        if (timeMoment.isValid()) {
          onValueChange(timeMoment.format(valueFormat));
        } else {
          onValueChange(null);
        }
      }
    };

    var _this$props3 = _this.props,
        _value = _this$props3.value,
        _valueFormat = _this$props3.valueFormat;
    var initialTime = moment(_value || '', _valueFormat);

    if (!initialTime.isValid()) {
      initialTime = moment();
    }

    _this.state = {
      selectedTime: initialTime.toDate()
    };
    return _this;
  }

  _createClass(TimePicker, [{
    key: "render",
    value: function render() {
      var _this2 = this;

      var _this$props4 = this.props,
          accessibilityLabel = _this$props4.accessibilityLabel,
          autoFocus = _this$props4.autoFocus,
          clearable = _this$props4.clearable,
          clearValueText = _this$props4.clearValueText,
          onBlur = _this$props4.onBlur,
          onFocus = _this$props4.onFocus,
          readOnly = _this$props4.readOnly,
          renderDropdownIcon = _this$props4.renderDropdownIcon,
          style = _this$props4.style,
          testID = _this$props4.testID;
      var styleProps = {
        style: style
      };
      var props = {
        accessibilityLabel: accessibilityLabel,
        autoFocus: autoFocus,
        clearable: clearable,
        clearValueText: clearValueText,
        onBlur: onBlur,
        onClear: function onClear() {
          return _this2._selectTime(null);
        },
        onConfirm: function onConfirm() {
          return _this2._selectTime(_this2.state.selectedTime || getDefaultTime());
        },
        onFocus: onFocus,
        readOnly: readOnly,
        ref: this._capturePickerRef,
        renderDropdownIcon: renderDropdownIcon,
        renderPopup: this._renderPopup,
        renderValue: this._renderValue,
        testID: testID
      };
      return /*#__PURE__*/React.createElement(PickerNative, Object.assign({}, styleProps, props));
    }
  }]);

  return TimePicker;
}(React.Component);

TimePicker.defaultProps = {
  clearable: true,
  clearValueText: 'Clear value',
  displayFormat: DEFAULT_TIME_DISPLAY_FORMAT,
  readOnly: false,
  valueFormat: DEFAULT_TIME_VALUE_FORMAT
};
TimePicker.displayName = 'TimePicker';
export default TimePicker;