import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";

/* eslint-disable react/no-array-index-key */
import moment from 'moment';
import * as React from 'react';
import styled from 'styled-components';
import Text from '../../../primitives/Text';
import { HOURS, MERIDIEMS, MINUTES } from '../constants';
var CELL_HEIGHT = 25;

var WheelPicker = /*#__PURE__*/function (_React$Component) {
  _inherits(WheelPicker, _React$Component);

  var _super = _createSuper(WheelPicker);

  function WheelPicker(props) {
    var _this;

    _classCallCheck(this, WheelPicker);

    _this = _super.call(this, props);
    _this._wheelRefs = [];

    _this._submit = function () {
      var onSelect = _this.props.onSelect;
      var _this$state = _this.state,
          selectedHour = _this$state.selectedHour,
          selectedMinute = _this$state.selectedMinute,
          selectedMeridiem = _this$state.selectedMeridiem;
      var hour = selectedMeridiem === 'AM' ? selectedHour : selectedHour + 12;
      var value = moment({
        hour: hour,
        minute: selectedMinute
      });
      onSelect && onSelect(value);
    };

    _this._renderWheels = function () {
      return /*#__PURE__*/React.createElement(WheelsContainer, null, /*#__PURE__*/React.createElement(Wheel, {
        ref: function ref(_ref) {
          _this._wheelRefs[0] = _ref;
        }
      }, HOURS.map(function (hour, hourNum) {
        var isSelected = hourNum === _this.state.selectedHour;
        return /*#__PURE__*/React.createElement(Cell, {
          key: hour,
          onClick: function onClick() {
            return _this.setState({
              selectedHour: hourNum
            });
          },
          $isSelected: isSelected
        }, /*#__PURE__*/React.createElement(CellText, {
          $isSelected: isSelected
        }, hour));
      })), /*#__PURE__*/React.createElement(Wheel, {
        ref: function ref(_ref2) {
          _this._wheelRefs[1] = _ref2;
        }
      }, MINUTES.map(function (minute, minuteNum) {
        var isSelected = minuteNum === _this.state.selectedMinute;
        return /*#__PURE__*/React.createElement(Cell, {
          key: minuteNum,
          onClick: function onClick() {
            return _this.setState({
              selectedMinute: minuteNum
            });
          },
          $isSelected: isSelected
        }, /*#__PURE__*/React.createElement(CellText, {
          $isSelected: isSelected
        }, minute));
      })), /*#__PURE__*/React.createElement(Wheel, {
        ref: function ref(_ref3) {
          _this._wheelRefs[2] = _ref3;
        }
      }, MERIDIEMS.map(function (meridiem) {
        var isSelected = meridiem === _this.state.selectedMeridiem;
        return /*#__PURE__*/React.createElement(Cell, {
          key: meridiem,
          onClick: function onClick() {
            return _this.setState({
              selectedMeridiem: meridiem
            });
          },
          $isSelected: isSelected
        }, /*#__PURE__*/React.createElement(CellText, {
          $isSelected: isSelected
        }, meridiem));
      })));
    };

    _this._renderFooter = function () {
      return /*#__PURE__*/React.createElement(FooterContainer, null, /*#__PURE__*/React.createElement(DoneButton, {
        onClick: _this._submit
      }, /*#__PURE__*/React.createElement(Text, null, "Done")));
    };

    var initialTime = _this.props.initialTime;

    if (!initialTime || !initialTime.isValid()) {
      initialTime = moment();
    }

    _this.state = {
      selectedHour: initialTime.hour() % 12,
      selectedMinute: initialTime.minute(),
      selectedMeridiem: initialTime.format('A')
    };
    return _this;
  }

  _createClass(WheelPicker, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this$state2 = this.state,
          selectedHour = _this$state2.selectedHour,
          selectedMinute = _this$state2.selectedMinute;

      if (selectedHour > 4 && this._wheelRefs[0]) {
        this._wheelRefs[0].scrollTop = CELL_HEIGHT * (selectedHour - 4);
      }

      if (selectedMinute > 4 && this._wheelRefs[1]) {
        this._wheelRefs[1].scrollTop = CELL_HEIGHT * (selectedMinute - 4);
      }
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement(Container, {
        $displayRight: this.props.displayRight
      }, this._renderWheels(), this._renderFooter());
    }
  }]);

  return WheelPicker;
}(React.Component);

export default WheelPicker;
var Container = styled.div.attrs(function (props) {
  return {
    'data-component': 'wheel-picker'
  };
}).withConfig({
  displayName: "WheelPicker__Container",
  componentId: "sc-18gpl15-0"
})(["", ""], function (_ref4) {
  var $displayRight = _ref4.$displayRight;
  return $displayRight && "\n    position: absolute;\n    border: 1px solid #ccc;\n    right: ".concat($displayRight, "px;\n    background-color: #fff;\n  ");
});
var Cell = styled.div.withConfig({
  displayName: "WheelPicker__Cell",
  componentId: "sc-18gpl15-1"
})(["align-items:center;cursor:pointer;display:flex;height:", "px;justify-content:center;padding:0 25px;", ";"], CELL_HEIGHT, function (_ref5) {
  var $isSelected = _ref5.$isSelected;
  return $isSelected ? "\n    background-color: #445;\n  " : "\n    &:hover {\n      background-color: #eee;\n    }\n  ";
});
var CellText = styled(Text).withConfig({
  displayName: "WheelPicker__CellText",
  componentId: "sc-18gpl15-2"
})(["", ";"], function (_ref6) {
  var $isSelected = _ref6.$isSelected;
  return $isSelected && "color: white;";
});
var DoneButton = styled.button.attrs(function (props) {
  return {
    tabIndex: -1,
    type: 'button'
  };
}).withConfig({
  displayName: "WheelPicker__DoneButton",
  componentId: "sc-18gpl15-3"
})(["background:none;border:none;cursor:pointer;height:100%;width:100%;&:hover{background-color:#eee;}"]);
var FooterContainer = styled.div.withConfig({
  displayName: "WheelPicker__FooterContainer",
  componentId: "sc-18gpl15-4"
})(["border-top:1px solid #ddd;height:25px;"]);
var Wheel = styled.div.withConfig({
  displayName: "WheelPicker__Wheel",
  componentId: "sc-18gpl15-5"
})(["overflow-y:auto;width:70px;&:not(:last-child){border-right:1px solid #ddd;}"]);
var WheelsContainer = styled.div.withConfig({
  displayName: "WheelPicker__WheelsContainer",
  componentId: "sc-18gpl15-6"
})(["display:flex;height:", "px;"], 9 * CELL_HEIGHT);