import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import React from 'react';
import warning from 'warning';
import { useFieldState } from '../../../primitives/Form';
import OldSelect from '../../OldSelect';
import Select from '../../Select';
import { InputFieldWrapper } from '../wrappers';

function SelectField(props, ref) {
  var passedFieldState = props.fieldState,
      label = props.label,
      modal = props.modal,
      options = props.options,
      required = props.required,
      reserveErrorSpace = props.reserveErrorSpace,
      testID = props.testID,
      validate = props.validate,
      rest = _objectWithoutProperties(props, ["fieldState", "label", "modal", "options", "required", "reserveErrorSpace", "testID", "validate"]);

  warning(!(validate && !reserveErrorSpace), "Prop \"reserveErrorSpace\" is set to false while \"validate\" is supplied. You will not see the validation error.");
  var SelectComponent = modal ? Select : OldSelect;
  var inputRef = React.useRef(null);
  var focusInput = React.useCallback(function () {
    return inputRef.current && inputRef.current.focus();
  }, []);
  var blurInput = React.useCallback(function () {
    return inputRef.current && inputRef.current.blur();
  }, []);
  React.useImperativeHandle(ref, function () {
    return {
      focus: focusInput,
      blur: blurInput
    };
  });

  var _useFieldState = useFieldState(props),
      fieldState = _useFieldState.fieldState,
      onBlur = _useFieldState.onBlur,
      onFocus = _useFieldState.onFocus,
      onValueChange = _useFieldState.onValueChange;

  return /*#__PURE__*/React.createElement(InputFieldWrapper, {
    fieldState: fieldState,
    focusInput: focusInput,
    label: label,
    onValueChange: onValueChange,
    required: required,
    reserveErrorSpace: reserveErrorSpace,
    testID: testID
  }, /*#__PURE__*/React.createElement(SelectComponent, Object.assign({}, rest, {
    ref: inputRef,
    onBlur: onBlur,
    onFocus: onFocus,
    onValueChange: onValueChange,
    options: options,
    value: fieldState.value
  })));
}

var ForwardRefSelectField = React.forwardRef(SelectField); // Cast to NamedExoticComponent to remove `propTypes` which causes problems when
// type checking.

export default ForwardRefSelectField;