import throttle from 'lodash/throttle';
import * as React from 'react';
import styled from 'styled-components';
import { DivViewBase } from './ViewBase';
export var ScrollViewBase = React.forwardRef(function (_ref, ref) {
  var accessibilityLabel = _ref.accessibilityLabel,
      children = _ref.children,
      className = _ref.className,
      horizontal = _ref.horizontal,
      onLayout = _ref.onLayout,
      style = _ref.style,
      contentContainerStyle = _ref.contentContainerStyle,
      testID = _ref.testID,
      onScroll = _ref.onScroll,
      onContentSizeChange = _ref.onContentSizeChange,
      _ref$scrollEventThrot = _ref.scrollEventThrottle,
      scrollEventThrottle = _ref$scrollEventThrot === void 0 ? 16 : _ref$scrollEventThrot;
  var styleProps = {
    className: className,
    $horizontal: horizontal,
    style: style
  };

  var onContentLayout = function onContentLayout(event) {
    var _event$nativeEvent$la = event.nativeEvent.layout,
        width = _event$nativeEvent$la.width,
        height = _event$nativeEvent$la.height;
    onContentSizeChange && onContentSizeChange(width, height);
  };

  var _onScroll = function _onScroll(event) {
    event.persist();
    onScrollThrottled(event);
  };

  var onScrollThrottled = throttle(function (event) {
    onScroll && onScroll({
      nativeEvent: {
        contentOffset: {
          x: event.currentTarget.scrollLeft,
          y: event.currentTarget.scrollTop
        },
        contentSize: {
          height: event.currentTarget.scrollHeight,
          width: event.currentTarget.scrollWidth
        },
        layoutMeasurement: {
          height: event.currentTarget.offsetHeight,
          width: event.currentTarget.offsetWidth
        }
      },
      timeStamp: Date.now()
    });
  }, scrollEventThrottle);
  return /*#__PURE__*/React.createElement(ScrollContainer, Object.assign({}, styleProps, {
    accessibilityLabel: accessibilityLabel,
    ref: ref,
    onLayout: onLayout,
    onScroll: _onScroll,
    testID: testID
  }), /*#__PURE__*/React.createElement(ContentContainer, {
    $horizontal: horizontal,
    onLayout: onContentLayout,
    style: contentContainerStyle
  }, children));
});
var ScrollContainer = styled(DivViewBase).withConfig({
  displayName: "ScrollViewBase__ScrollContainer",
  componentId: "z5ujgb-0"
})(["flex:1;overflow-x:hidden;overflow-y:auto;-webkit-overflow-scrolling:touch;transform:translateZ(0);", ";"], function (_ref2) {
  var $horizontal = _ref2.$horizontal;
  return $horizontal && "\n    flex-direction: row;\n    overflow-x: auto;\n    overflow-y: hidden;\n  ";
});
var ContentContainer = styled(DivViewBase).withConfig({
  displayName: "ScrollViewBase__ContentContainer",
  componentId: "z5ujgb-1"
})(["", ";"], function (_ref3) {
  var $horizontal = _ref3.$horizontal;
  return $horizontal ? "\n    flex-direction: row;\n    min-width: 100%;\n  " : "\n    min-height: 100%;\n  ";
});