import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _unset from "lodash/fp/unset";
import _set from "lodash/fp/set";
import _pick from "lodash/fp/pick";
import _merge from "lodash/fp/merge";
import _identity from "lodash/fp/identity";
import _get from "lodash/fp/get";
import _flow from "lodash/fp/flow";
import * as React from 'react';
import { NoUpdate, ReComponent, Update } from 'react-recomponent';
import { FormStateProvider } from './Context';
import SubmissionError from './SubmissionError';
import { deepEqual, unwrapChildren } from './utils';
var formStateKeys = ['activeField', 'allTouched', 'anyTouched', 'fieldErrors', 'fieldStates', 'formError', 'submitting', 'values'];

var getFormState = function getFormState(state) {
  return _pick(formStateKeys, state);
};

var getStateAndHelpers = function getStateAndHelpers(state) {
  var allTouched = state.allTouched,
      fieldErrors = state.fieldErrors,
      fieldStates = state.fieldStates,
      initialValues = state.initialValues,
      values = state.values;
  var pristine = deepEqual(initialValues, values);

  var getFieldError = function getFieldError(fieldName) {
    return _get(fieldName, fieldErrors);
  };

  var getFieldValue = function getFieldValue(fieldName) {
    if (typeof values === 'object') {
      return _get(fieldName, values);
    }

    return undefined;
  };

  var fieldActive = function fieldActive(fieldName) {
    return !!_get([fieldName, 'active'], fieldStates);
  };

  var fieldPristine = function fieldPristine(fieldName) {
    var initialValue = typeof initialValues === 'object' ? _get(fieldName, initialValues) : undefined;
    return deepEqual(initialValue, getFieldValue(fieldName));
  };

  var fieldTouched = function fieldTouched(fieldName) {
    return !!allTouched || !!_get([fieldName, 'touched'], fieldStates);
  };

  var fieldVisited = function fieldVisited(fieldName) {
    return !!_get([fieldName, 'visited'], fieldStates);
  };

  return _objectSpread(_objectSpread({}, getFormState(state)), {}, {
    // computed
    dirty: !pristine,
    pristine: pristine,
    // helpers
    fieldActive: fieldActive,
    fieldDirty: function fieldDirty(fieldName) {
      return !fieldPristine(fieldName);
    },
    fieldPristine: fieldPristine,
    fieldTouched: fieldTouched,
    fieldVisited: fieldVisited,
    getFieldError: getFieldError,
    getFieldValue: getFieldValue
  });
};

var validate = function validate(state) {
  var stateAndHelpers = getStateAndHelpers(state);
  var fieldValidators = state.fieldValidators,
      formValidator = state.formValidator;
  var values = stateAndHelpers.values,
      getFieldValue = stateAndHelpers.getFieldValue;
  var errors = {}; // validate fields

  var validateField = function validateField(fieldName, fn) {
    if (typeof fn !== 'function') return;
    var error = fn(getFieldValue(fieldName), values);

    if (error) {
      errors = _set(fieldName, error, errors);
    }
  };

  Object.entries(fieldValidators).forEach(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        fieldName = _ref2[0],
        validators = _ref2[1];

    if (Array.isArray(validators)) {
      validators.forEach(function (validator) {
        return validateField(fieldName, validator);
      });
    } else {
      validateField(fieldName, validators);
    }
  }); // validate form

  if (formValidator) {
    errors = _merge(errors, formValidator(stateAndHelpers));
  }

  return errors;
};

var getStateAfterValidation = function getStateAfterValidation(state) {
  var _validate = validate(state),
      __formError = _validate.__formError,
      fieldErrors = _objectWithoutProperties(_validate, ["__formError"]);

  var newState = _set('fieldErrors', fieldErrors, state);

  if (__formError !== undefined) {
    newState = _set('formError', __formError, newState);
  }

  return newState;
};

var FormStateManager = /*#__PURE__*/function (_ReComponent) {
  _inherits(FormStateManager, _ReComponent);

  var _super = _createSuper(FormStateManager);

  function FormStateManager() {
    var _this;

    _classCallCheck(this, FormStateManager);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.state = {
      activeField: undefined,
      allTouched: false,
      anyTouched: false,
      fieldErrors: {},
      fieldStates: {},
      formError: undefined,
      submitting: false,
      values: _this.props.initialValues,
      // internal state
      fieldValidators: {},
      formValidator: undefined,
      initialValues: _this.props.initialValues
    };
    return _this;
  }

  _createClass(FormStateManager, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.send({
        type: 'VALIDATE'
      });
    }
  }, {
    key: "render",
    value: function render() {
      var render = unwrapChildren(this.props.children);

      if (typeof render !== 'function') {
        throw new Error('Prop `children` must be a function');
      }

      var stateAndHelpers = getStateAndHelpers(this.state);

      var stateAndHelpersAndActions = _objectSpread(_objectSpread({}, stateAndHelpers), {}, {
        actions: this._getActions(stateAndHelpers)
      });

      return /*#__PURE__*/React.createElement(FormStateProvider, {
        value: stateAndHelpersAndActions
      }, render(stateAndHelpersAndActions));
    }
  }, {
    key: "_getActions",
    value: function _getActions(stateAndHelpers) {
      var _this2 = this;

      return {
        blur: function blur(fieldName) {
          return _this2.send({
            type: 'BLUR',
            meta: {
              fieldName: fieldName
            }
          });
        },
        changeValue: function changeValue(fieldName, value) {
          _this2.send({
            type: 'CHANGE_VALUE',
            meta: {
              fieldName: fieldName
            },
            payload: value
          });
        },
        focus: function focus(fieldName) {
          return _this2.send({
            type: 'FOCUS',
            meta: {
              fieldName: fieldName
            }
          });
        },
        // setErrors: errors => {
        //   this.send({
        //     type: "SET_ERRORS",
        //     payload: errors
        //   });
        // },
        setFieldValidators: function setFieldValidators(fieldName, validators) {
          _this2.send({
            type: 'SET_FIELD_VALIDATORS',
            meta: {
              fieldName: fieldName
            },
            payload: validators
          });
        },
        submit: function () {
          var _submit = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(doSubmit) {
            return _regeneratorRuntime.wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    _context.prev = 0;

                    _this2.send({
                      type: 'START_SUBMIT'
                    });

                    _context.next = 4;
                    return doSubmit();

                  case 4:
                    _this2.send({
                      type: 'SET_SUBMIT_SUCCEEDED'
                    });

                    _context.next = 14;
                    break;

                  case 7:
                    _context.prev = 7;
                    _context.t0 = _context["catch"](0);

                    if (!(_context.t0 instanceof SubmissionError)) {
                      _context.next = 13;
                      break;
                    }

                    _this2.send({
                      type: 'SET_SUBMIT_FAILED',
                      payload: _context.t0.errors
                    });

                    _context.next = 14;
                    break;

                  case 13:
                    throw _context.t0;

                  case 14:
                  case "end":
                    return _context.stop();
                }
              }
            }, _callee, null, [[0, 7]]);
          }));

          function submit(_x) {
            return _submit.apply(this, arguments);
          }

          return submit;
        }(),
        touchAll: function touchAll() {
          _this2.send({
            type: 'TOUCH_ALL'
          });
        }
      };
    }
  }], [{
    key: "reducer",
    value: function reducer( // $FlowFixMe: ReComponent's action param is { type: ActionTypes }
    action, state) {
      switch (action.type) {
        case 'BLUR':
          {
            var fieldName = action.meta.fieldName;

            var update = _flow(state.activeField === fieldName ? _set('activeField', undefined) : _identity, _set('anyTouched', true), _unset(['fieldStates', fieldName, 'active']), state.allTouched ? _identity : _set(['fieldStates', fieldName, 'touched'], true));

            var newState = update(state);
            return Update(newState);
          }

        case 'CHANGE_VALUE':
          {
            var _fieldName = action.meta.fieldName,
                payload = action.payload;

            var _newState = _set("values.".concat(_fieldName), payload, state);

            _newState = getStateAfterValidation(_newState);
            return Update(_newState);
          }

        case 'FOCUS':
          {
            var _fieldName2 = action.meta.fieldName;

            var previouslyActive = _get('activeField', state);

            var _update = _flow(_set('activeField', _fieldName2), _unset(['fieldStates', previouslyActive, 'active']), _set(['fieldStates', _fieldName2, 'active'], true), _set(['fieldStates', _fieldName2, 'visited'], true));

            var _newState2 = _update(state);

            return Update(_newState2);
          }
        // case "SET_ERRORS": {
        //   const { payload: { __formError, ...fieldErrors } } = action;
        //   const update = flow(
        //     set("fieldErrors", fieldErrors),
        //     set("formError", __formError)
        //   );
        //   const newState = update(state);
        //   return Update(newState);
        // }

        case 'SET_FIELD_VALIDATORS':
          {
            var _fieldName3 = action.meta.fieldName,
                _payload = action.payload;

            var _newState3 = _set(['fieldValidators', _fieldName3], _payload, state); // validate


            _newState3 = getStateAfterValidation(_newState3);
            return Update(_newState3);
          }

        case 'SET_SUBMIT_SUCCEEDED':
          {
            var _update2 = _flow(_set('submitting', false), _set('formError', undefined) // unset("submitFailed"),
            // set("submitSucceeded", true),
            // unset("submitErrors")
            );

            var _newState4 = _update2(state);

            return Update(_newState4);
          }

        case 'SET_SUBMIT_FAILED':
          {
            var _action$payload = action.payload,
                __formError = _action$payload.__formError,
                fieldErrors = _objectWithoutProperties(_action$payload, ["__formError"]);

            var _update3 = _flow(_set('submitting', false), _set('fieldErrors', fieldErrors), _set('formError', __formError) // unset('submitSucceeded'),
            // set('submitFailed', true),
            // Object.keys(fieldErrors).length ? set('submitErrors', fieldErrors) : unset('submitErrors'),
            );

            var _newState5 = _update3(state);

            return Update(_newState5);
          }

        case 'START_SUBMIT':
          {
            var _newState6 = _set('submitting', true, state);

            return Update(_newState6);
          }

        case 'TOUCH_ALL':
          {
            var _update4 = _flow(_set('allTouched', true), _set('anyTouched', true));

            var _newState7 = _update4(state);

            return Update(_newState7);
          }

        case 'VALIDATE':
          {
            var _newState8 = getStateAfterValidation(state);

            return Update(_newState8);
          }

        default:
          return NoUpdate();
      }
    }
  }]);

  return FormStateManager;
}(ReComponent);

FormStateManager.defaultProps = {
  initialValues: {}
};
export default FormStateManager;