import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _set from "lodash/fp/set";
import _get from "lodash/fp/get";
import pdfMake from 'pdfmake';
import InteractionManager from '../../../internals/InteractionManager';
var filePathSymbol = Symbol('pdfFilePath');

var getStandardFontFileDefinition = function getStandardFontFileDefinition(fileName) {
  return _defineProperty({
    fileName: fileName,
    url: "https://unpkg.com/pdfkit@0.10.0/js/data/".concat(fileName, ".afm")
  }, filePathSymbol, "data/".concat(fileName, ".afm"));
};

var fontDefinitions = {
  Courier: {
    normal: getStandardFontFileDefinition('Courier'),
    bold: getStandardFontFileDefinition('Courier-Bold'),
    italics: getStandardFontFileDefinition('Courier-Oblique'),
    bolditalics: getStandardFontFileDefinition('Courier-BoldOblique')
  },
  Helvetica: {
    normal: getStandardFontFileDefinition('Helvetica'),
    bold: getStandardFontFileDefinition('Helvetica-Bold'),
    italics: getStandardFontFileDefinition('Helvetica-Oblique'),
    bolditalics: getStandardFontFileDefinition('Helvetica-BoldOblique')
  },
  Times: {
    normal: getStandardFontFileDefinition('Times-Roman'),
    bold: getStandardFontFileDefinition('Times-Bold'),
    italics: getStandardFontFileDefinition('Times-Italic'),
    bolditalics: getStandardFontFileDefinition('Times-BoldItalic')
  },
  Symbol: {
    normal: getStandardFontFileDefinition('Symbol')
  },
  ZapfDingbats: {
    normal: getStandardFontFileDefinition('ZapfDingbats')
  }
};
var defaultFont = {
  name: 'Helvetica',
  style: 'normal'
};
/**
 * Save font data to pdfMake's virtual file system
 */

function saveFontFile(filePath, data) {
  if (!pdfMake.vfs) {
    pdfMake.vfs = {};
  }

  pdfMake.vfs[filePath] = data;
}
/**
 * Fetch font file and register with pdfmake
 */


function fetchFontFile(_x, _x2, _x3) {
  return _fetchFontFile.apply(this, arguments);
}

function _fetchFontFile() {
  _fetchFontFile = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(fontName, fontStyle, fontFileDefinition) {
    var data, fontPath;
    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return new Promise(function (resolve, reject) {
              var request = new XMLHttpRequest();
              request.open('GET', fontFileDefinition.url, true); // get afm files for standard fonts as text and custom fonts as arraybuffer

              var isStandard = !!fontFileDefinition[filePathSymbol];
              request.responseType = isStandard ? 'text' : 'arraybuffer';

              request.onload = function (e) {
                if (request.status === 200) {
                  resolve(request.response);
                } else {
                  reject(new Error("Fetching \"".concat(fontFileDefinition.url, "\" failed: ").concat(request.statusText)));
                }
              };

              request.onerror = function (error) {
                return new Error("Fetching \"".concat(fontFileDefinition.url, "\" failed: ").concat(error));
              };

              request.send();
            });

          case 2:
            data = _context3.sent;
            fontPath = fontFileDefinition[filePathSymbol] || fontFileDefinition.fileName;
            saveFontFile(fontPath, data); // register font with pdfmake

            pdfMake.fonts[fontName][fontStyle] = fontFileDefinition.fileName;

          case 6:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _fetchFontFile.apply(this, arguments);
}

var MAX_RETRIES = 3;

function fetchFontFileWithRetry(_x4, _x5, _x6) {
  return _fetchFontFileWithRetry.apply(this, arguments);
}

function _fetchFontFileWithRetry() {
  _fetchFontFileWithRetry = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(fontName, fontStyle, fontFileDefinition) {
    var numOfAttempts;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            numOfAttempts = 0;

          case 1:
            if (!true) {
              _context4.next = 22;
              break;
            }

            _context4.prev = 2;
            _context4.next = 5;
            return fetchFontFile(fontName, fontStyle, fontFileDefinition);

          case 5:
            return _context4.abrupt("return");

          case 8:
            _context4.prev = 8;
            _context4.t0 = _context4["catch"](2);
            numOfAttempts += 1;

            if (!(numOfAttempts >= MAX_RETRIES)) {
              _context4.next = 18;
              break;
            }

            if (!(fontName === defaultFont.name && fontStyle === defaultFont.style)) {
              _context4.next = 16;
              break;
            }

            throw _context4.t0;

          case 16:
            // not throwing error and just use the default font as a fallback
            console.warn("Failed to fetch font ".concat(fontName, " (").concat(fontStyle, "). Falling back to ").concat(defaultFont.name, " (").concat(defaultFont.style, ")."));
            return _context4.abrupt("return");

          case 18:
            _context4.next = 20;
            return new Promise(function (resolve) {
              return setTimeout(resolve, 1000);
            });

          case 20:
            _context4.next = 1;
            break;

          case 22:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, null, [[2, 8]]);
  }));
  return _fetchFontFileWithRetry.apply(this, arguments);
}

var createFileManager = function createFileManager() {
  var fileStatusMap = {};
  var allFontFilesSettled = false;
  var resolveAllFilesSettled;
  var allFontFilesSettledPromise = new Promise(function (resolve) {
    resolveAllFilesSettled = function resolveAllFilesSettled() {
      allFontFilesSettled = true;
      resolve();
    };
  });

  var tryResolveAllFilesSettled = function tryResolveAllFilesSettled() {
    if (Object.values(fileStatusMap).every(function (status) {
      return status === 'fulfilled' || status === 'rejected';
    })) {
      resolveAllFilesSettled();
    }
  };

  return {
    add: function add(fileName, fetchFile) {
      if (fileStatusMap[fileName]) {
        // file already loaded
        return;
      } // reset promise if it's already settled


      if (allFontFilesSettled) {
        allFontFilesSettledPromise = new Promise(function (resolve) {
          resolveAllFilesSettled = function resolveAllFilesSettled() {
            allFontFilesSettled = true;
            resolve();
          };
        });
      }

      fileStatusMap[fileName] = 'pending';
      fetchFile().then(function () {
        fileStatusMap[fileName] = 'fulfilled';
        tryResolveAllFilesSettled();
      })["catch"](function (err) {
        console.error(err);
        fileStatusMap[fileName] = 'rejected';
        tryResolveAllFilesSettled();
      });
    },
    waitForAll: function waitForAll() {
      return _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return allFontFilesSettledPromise;

              case 2:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }))();
    }
  };
};

var fileManager = createFileManager();

function loadFontIfNecessary(fontName, fontStyle, fontFileDefinition) {
  var fileName = fontFileDefinition.fileName;

  if (!pdfMake.fonts[fontName]) {
    // point to the default font so that if the font failed to load we can still
    // show something instead of crashing
    pdfMake.fonts[fontName] = {
      normal: fontDefinitions[defaultFont.name][defaultFont.style].fileName
    };
  }

  fileManager.add(fileName, function () {
    return fetchFontFileWithRetry(fontName, fontStyle, fontFileDefinition);
  });
}

export function initializePdfFonts() {
  if (!pdfMake.fonts) {
    pdfMake.fonts = {};
  } // load all font files if not loaded yet


  InteractionManager.runAfterInteractions( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            Object.entries(fontDefinitions).forEach(function (_ref3) {
              var _ref4 = _slicedToArray(_ref3, 2),
                  fontName = _ref4[0],
                  fontDefinition = _ref4[1];

              Object.entries(fontDefinition).forEach(function (_ref5) {
                var _ref6 = _slicedToArray(_ref5, 2),
                    fontStyle = _ref6[0],
                    fontFileDefinition = _ref6[1];

                loadFontIfNecessary(fontName, fontStyle, fontFileDefinition);
              });
            });

          case 1:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  })));
}

function sanitizeDefinition(definition) {
  if (typeof definition !== 'object' || definition === 'null') {
    throw new Error("Pdf: Invalid pdf definition");
  }

  var fontPropertyPath = 'defaultStyle.font';

  if (_get(fontPropertyPath, definition)) {
    return definition;
  }

  return _set(fontPropertyPath, defaultFont.name, definition);
}

function sanitizeTableLayouts(tableLayouts) {
  if (tableLayouts === undefined) return tableLayouts;

  if (typeof tableLayouts !== 'object' || tableLayouts === 'null') {
    throw new Error("Pdf: Invalid table layouts");
  }

  return tableLayouts;
}

export function createDataUrlFromDefinition(_x7) {
  return _createDataUrlFromDefinition.apply(this, arguments);
}

function _createDataUrlFromDefinition() {
  _createDataUrlFromDefinition = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(_ref7) {
    var definition, fonts, tableLayouts, sanitizedDefinition, sanitizedTableLayouts, pdfDocGenerator;
    return _regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            definition = _ref7.definition, fonts = _ref7.fonts, tableLayouts = _ref7.tableLayouts;
            sanitizedDefinition = sanitizeDefinition(definition);
            sanitizedTableLayouts = sanitizeTableLayouts(tableLayouts); // load custom fonts

            if (fonts) {
              Object.entries(fonts).forEach(function (_ref8) {
                var _ref9 = _slicedToArray(_ref8, 2),
                    fontName = _ref9[0],
                    fontDefinition = _ref9[1];

                Object.entries(fontDefinition).forEach(function (_ref10) {
                  var _ref11 = _slicedToArray(_ref10, 2),
                      fontStyle = _ref11[0],
                      url = _ref11[1];

                  if (url) {
                    loadFontIfNecessary(fontName, fontStyle, {
                      fileName: "".concat(fontName, "-").concat(fontStyle),
                      url: url
                    });
                  }
                });
              });
            } // TODO: only wait for fonts used by the definition


            _context5.next = 6;
            return fileManager.waitForAll();

          case 6:
            pdfDocGenerator = pdfMake.createPdf(sanitizedDefinition, sanitizedTableLayouts);
            return _context5.abrupt("return", new Promise(function (resolve) {
              pdfDocGenerator.getDataUrl(resolve);
            }));

          case 8:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return _createDataUrlFromDefinition.apply(this, arguments);
}