import * as React from 'react';
import styled from 'styled-components';
import { DivViewBase } from '../../../internals/web/ViewBase';
var Switch = React.forwardRef(function (_ref, ref) {
  var accessibilityLabel = _ref.accessibilityLabel,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      onBlur = _ref.onBlur,
      onFocus = _ref.onFocus,
      onValueChange = _ref.onValueChange,
      style = _ref.style,
      testID = _ref.testID,
      _ref$value = _ref.value,
      value = _ref$value === void 0 ? false : _ref$value,
      className = _ref.className;
  var containerRef = React.useRef(null);
  React.useImperativeHandle(ref, function () {
    return {
      focus: function focus() {
        var _containerRef$current;

        return (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.focus();
      },
      blur: function blur() {
        var _containerRef$current2;

        return (_containerRef$current2 = containerRef.current) === null || _containerRef$current2 === void 0 ? void 0 : _containerRef$current2.blur();
      }
    };
  });

  var _onBlur = function _onBlur(event) {
    onBlur === null || onBlur === void 0 ? void 0 : onBlur({
      nativeEvent: {
        value: value
      }
    });
  };

  var _onFocus = function _onFocus(event) {
    onFocus === null || onFocus === void 0 ? void 0 : onFocus({
      nativeEvent: {
        value: value
      }
    });
  };

  var _onKeyPress = function _onKeyPress(event) {
    var ENTER = 13;
    var SPACE = 32;

    if (event.which === ENTER || event.which === SPACE) {
      event.preventDefault();
      event.stopPropagation();

      _toggle();
    }
  };

  var _toggle = function _toggle() {
    return !disabled && (onValueChange === null || onValueChange === void 0 ? void 0 : onValueChange(!value));
  };

  var props = {
    accessibilityLabel: accessibilityLabel,
    onBlur: _onBlur,
    onClick: _toggle,
    onFocus: _onFocus,
    onKeyPress: _onKeyPress,
    testID: testID
  };
  var styleProps = {
    className: className,
    style: style,
    $value: !!value
  };
  return /*#__PURE__*/React.createElement(SwitchContainer, Object.assign({
    ref: containerRef,
    $disabled: disabled
  }, props, styleProps), /*#__PURE__*/React.createElement(Track, {
    $value: !!value
  }), /*#__PURE__*/React.createElement(FocusIndicator, {
    $disabled: disabled,
    $position: value ? 'right' : 'left'
  }, /*#__PURE__*/React.createElement(Handle, {
    $disabled: disabled
  })));
});
export default Switch;
var SwitchContainer = styled(DivViewBase).attrs(function (_ref2) {
  var $value = _ref2.$value;
  return {
    'aria-checked': $value ? 'true' : 'false',
    role: 'checkbox',
    tabIndex: 0
  };
}).withConfig({
  displayName: "Switch__SwitchContainer",
  componentId: "d6bfc9-0"
})(["outline:none;display:inline-flex;flex:none;cursor:", ";height:38px;width:56px;padding:9px;"], function (_ref3) {
  var $disabled = _ref3.$disabled;
  return "".concat(!$disabled ? 'pointer' : 'not-allowed');
});
var Handle = styled(DivViewBase).withConfig({
  displayName: "Switch__Handle",
  componentId: "d6bfc9-1"
})(["background-color:", ";border-radius:50%;height:20px;width:20px;box-shadow:0px 2px 1px -1px rgba(0,0,0,0.2),0px 1px 1px 0px rgba(0,0,0,0.14),0px 1px 3px 0px rgba(0,0,0,0.12);"], function (_ref4) {
  var $disabled = _ref4.$disabled;
  return "".concat(!$disabled ? 'white' : '#d6d3d1');
});
var FocusIndicator = styled(DivViewBase).withConfig({
  displayName: "Switch__FocusIndicator",
  componentId: "d6bfc9-2"
})(["background-color:transparent;border-radius:50%;padding:9px;position:absolute;top:0px;left:", "px;transition:left 0.2s,background-color 0.2s;", ""], function (_ref5) {
  var $position = _ref5.$position;
  return $position === 'left' ? 0 : 18;
}, function (_ref6) {
  var $disabled = _ref6.$disabled;
  return !$disabled ? "".concat(SwitchContainer, ":hover &, ").concat(SwitchContainer, ":focus &  {\n    background-color: #00000029;\n  },") : "";
});
var Track = styled(DivViewBase).withConfig({
  displayName: "Switch__Track",
  componentId: "d6bfc9-3"
})(["background-color:", ";border-radius:12px;width:100%;height:100%;"], function (_ref7) {
  var $value = _ref7.$value;
  return $value ? '#FF7800' : '#B0ADAB';
});