"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _unset2 = _interopRequireDefault(require("lodash/fp/unset"));

var _set2 = _interopRequireDefault(require("lodash/fp/set"));

var _pick2 = _interopRequireDefault(require("lodash/fp/pick"));

var _merge2 = _interopRequireDefault(require("lodash/fp/merge"));

var _identity2 = _interopRequireDefault(require("lodash/fp/identity"));

var _get2 = _interopRequireDefault(require("lodash/fp/get"));

var _flow2 = _interopRequireDefault(require("lodash/fp/flow"));

var React = _interopRequireWildcard(require("react"));

var _reactRecomponent = require("react-recomponent");

var _Context = require("./Context");

var _SubmissionError = _interopRequireDefault(require("./SubmissionError"));

var _utils = require("./utils");

var formStateKeys = ['activeField', 'allTouched', 'anyTouched', 'fieldErrors', 'fieldStates', 'formError', 'submitting', 'values'];

var getFormState = function getFormState(state) {
  return (0, _pick2["default"])(formStateKeys, state);
};

var getStateAndHelpers = function getStateAndHelpers(state) {
  var allTouched = state.allTouched,
      fieldErrors = state.fieldErrors,
      fieldStates = state.fieldStates,
      initialValues = state.initialValues,
      values = state.values;
  var pristine = (0, _utils.deepEqual)(initialValues, values);

  var getFieldError = function getFieldError(fieldName) {
    return (0, _get2["default"])(fieldName, fieldErrors);
  };

  var getFieldValue = function getFieldValue(fieldName) {
    if (typeof values === 'object') {
      return (0, _get2["default"])(fieldName, values);
    }

    return undefined;
  };

  var fieldActive = function fieldActive(fieldName) {
    return !!(0, _get2["default"])([fieldName, 'active'], fieldStates);
  };

  var fieldPristine = function fieldPristine(fieldName) {
    var initialValue = typeof initialValues === 'object' ? (0, _get2["default"])(fieldName, initialValues) : undefined;
    return (0, _utils.deepEqual)(initialValue, getFieldValue(fieldName));
  };

  var fieldTouched = function fieldTouched(fieldName) {
    return !!allTouched || !!(0, _get2["default"])([fieldName, 'touched'], fieldStates);
  };

  var fieldVisited = function fieldVisited(fieldName) {
    return !!(0, _get2["default"])([fieldName, 'visited'], fieldStates);
  };

  return (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, getFormState(state)), {}, {
    // computed
    dirty: !pristine,
    pristine: pristine,
    // helpers
    fieldActive: fieldActive,
    fieldDirty: function fieldDirty(fieldName) {
      return !fieldPristine(fieldName);
    },
    fieldPristine: fieldPristine,
    fieldTouched: fieldTouched,
    fieldVisited: fieldVisited,
    getFieldError: getFieldError,
    getFieldValue: getFieldValue
  });
};

var validate = function validate(state) {
  var stateAndHelpers = getStateAndHelpers(state);
  var fieldValidators = state.fieldValidators,
      formValidator = state.formValidator;
  var values = stateAndHelpers.values,
      getFieldValue = stateAndHelpers.getFieldValue;
  var errors = {}; // validate fields

  var validateField = function validateField(fieldName, fn) {
    if (typeof fn !== 'function') return;
    var error = fn(getFieldValue(fieldName), values);

    if (error) {
      errors = (0, _set2["default"])(fieldName, error, errors);
    }
  };

  Object.entries(fieldValidators).forEach(function (_ref) {
    var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
        fieldName = _ref2[0],
        validators = _ref2[1];

    if (Array.isArray(validators)) {
      validators.forEach(function (validator) {
        return validateField(fieldName, validator);
      });
    } else {
      validateField(fieldName, validators);
    }
  }); // validate form

  if (formValidator) {
    errors = (0, _merge2["default"])(errors, formValidator(stateAndHelpers));
  }

  return errors;
};

var getStateAfterValidation = function getStateAfterValidation(state) {
  var _validate = validate(state),
      __formError = _validate.__formError,
      fieldErrors = (0, _objectWithoutProperties2["default"])(_validate, ["__formError"]);

  var newState = (0, _set2["default"])('fieldErrors', fieldErrors, state);

  if (__formError !== undefined) {
    newState = (0, _set2["default"])('formError', __formError, newState);
  }

  return newState;
};

var FormStateManager = /*#__PURE__*/function (_ReComponent) {
  (0, _inherits2["default"])(FormStateManager, _ReComponent);

  var _super = (0, _createSuper2["default"])(FormStateManager);

  function FormStateManager() {
    var _this;

    (0, _classCallCheck2["default"])(this, FormStateManager);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.state = {
      activeField: undefined,
      allTouched: false,
      anyTouched: false,
      fieldErrors: {},
      fieldStates: {},
      formError: undefined,
      submitting: false,
      values: _this.props.initialValues,
      // internal state
      fieldValidators: {},
      formValidator: undefined,
      initialValues: _this.props.initialValues
    };
    return _this;
  }

  (0, _createClass2["default"])(FormStateManager, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.send({
        type: 'VALIDATE'
      });
    }
  }, {
    key: "render",
    value: function render() {
      var render = (0, _utils.unwrapChildren)(this.props.children);

      if (typeof render !== 'function') {
        throw new Error('Prop `children` must be a function');
      }

      var stateAndHelpers = getStateAndHelpers(this.state);
      var stateAndHelpersAndActions = (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, stateAndHelpers), {}, {
        actions: this._getActions(stateAndHelpers)
      });
      return /*#__PURE__*/React.createElement(_Context.FormStateProvider, {
        value: stateAndHelpersAndActions
      }, render(stateAndHelpersAndActions));
    }
  }, {
    key: "_getActions",
    value: function _getActions(stateAndHelpers) {
      var _this2 = this;

      return {
        blur: function blur(fieldName) {
          return _this2.send({
            type: 'BLUR',
            meta: {
              fieldName: fieldName
            }
          });
        },
        changeValue: function changeValue(fieldName, value) {
          _this2.send({
            type: 'CHANGE_VALUE',
            meta: {
              fieldName: fieldName
            },
            payload: value
          });
        },
        focus: function focus(fieldName) {
          return _this2.send({
            type: 'FOCUS',
            meta: {
              fieldName: fieldName
            }
          });
        },
        // setErrors: errors => {
        //   this.send({
        //     type: "SET_ERRORS",
        //     payload: errors
        //   });
        // },
        setFieldValidators: function setFieldValidators(fieldName, validators) {
          _this2.send({
            type: 'SET_FIELD_VALIDATORS',
            meta: {
              fieldName: fieldName
            },
            payload: validators
          });
        },
        submit: function () {
          var _submit = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(doSubmit) {
            return _regenerator["default"].wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    _context.prev = 0;

                    _this2.send({
                      type: 'START_SUBMIT'
                    });

                    _context.next = 4;
                    return doSubmit();

                  case 4:
                    _this2.send({
                      type: 'SET_SUBMIT_SUCCEEDED'
                    });

                    _context.next = 14;
                    break;

                  case 7:
                    _context.prev = 7;
                    _context.t0 = _context["catch"](0);

                    if (!(_context.t0 instanceof _SubmissionError["default"])) {
                      _context.next = 13;
                      break;
                    }

                    _this2.send({
                      type: 'SET_SUBMIT_FAILED',
                      payload: _context.t0.errors
                    });

                    _context.next = 14;
                    break;

                  case 13:
                    throw _context.t0;

                  case 14:
                  case "end":
                    return _context.stop();
                }
              }
            }, _callee, null, [[0, 7]]);
          }));

          function submit(_x) {
            return _submit.apply(this, arguments);
          }

          return submit;
        }(),
        touchAll: function touchAll() {
          _this2.send({
            type: 'TOUCH_ALL'
          });
        }
      };
    }
  }], [{
    key: "reducer",
    value: function reducer( // $FlowFixMe: ReComponent's action param is { type: ActionTypes }
    action, state) {
      switch (action.type) {
        case 'BLUR':
          {
            var fieldName = action.meta.fieldName;
            var update = (0, _flow2["default"])(state.activeField === fieldName ? (0, _set2["default"])('activeField', undefined) : _identity2["default"], (0, _set2["default"])('anyTouched', true), (0, _unset2["default"])(['fieldStates', fieldName, 'active']), state.allTouched ? _identity2["default"] : (0, _set2["default"])(['fieldStates', fieldName, 'touched'], true));
            var newState = update(state);
            return (0, _reactRecomponent.Update)(newState);
          }

        case 'CHANGE_VALUE':
          {
            var _fieldName = action.meta.fieldName,
                payload = action.payload;

            var _newState = (0, _set2["default"])("values.".concat(_fieldName), payload, state);

            _newState = getStateAfterValidation(_newState);
            return (0, _reactRecomponent.Update)(_newState);
          }

        case 'FOCUS':
          {
            var _fieldName2 = action.meta.fieldName;
            var previouslyActive = (0, _get2["default"])('activeField', state);

            var _update = (0, _flow2["default"])((0, _set2["default"])('activeField', _fieldName2), (0, _unset2["default"])(['fieldStates', previouslyActive, 'active']), (0, _set2["default"])(['fieldStates', _fieldName2, 'active'], true), (0, _set2["default"])(['fieldStates', _fieldName2, 'visited'], true));

            var _newState2 = _update(state);

            return (0, _reactRecomponent.Update)(_newState2);
          }
        // case "SET_ERRORS": {
        //   const { payload: { __formError, ...fieldErrors } } = action;
        //   const update = flow(
        //     set("fieldErrors", fieldErrors),
        //     set("formError", __formError)
        //   );
        //   const newState = update(state);
        //   return Update(newState);
        // }

        case 'SET_FIELD_VALIDATORS':
          {
            var _fieldName3 = action.meta.fieldName,
                _payload = action.payload;

            var _newState3 = (0, _set2["default"])(['fieldValidators', _fieldName3], _payload, state); // validate


            _newState3 = getStateAfterValidation(_newState3);
            return (0, _reactRecomponent.Update)(_newState3);
          }

        case 'SET_SUBMIT_SUCCEEDED':
          {
            var _update2 = (0, _flow2["default"])((0, _set2["default"])('submitting', false), (0, _set2["default"])('formError', undefined) // unset("submitFailed"),
            // set("submitSucceeded", true),
            // unset("submitErrors")
            );

            var _newState4 = _update2(state);

            return (0, _reactRecomponent.Update)(_newState4);
          }

        case 'SET_SUBMIT_FAILED':
          {
            var _action$payload = action.payload,
                __formError = _action$payload.__formError,
                fieldErrors = (0, _objectWithoutProperties2["default"])(_action$payload, ["__formError"]);

            var _update3 = (0, _flow2["default"])((0, _set2["default"])('submitting', false), (0, _set2["default"])('fieldErrors', fieldErrors), (0, _set2["default"])('formError', __formError) // unset('submitSucceeded'),
            // set('submitFailed', true),
            // Object.keys(fieldErrors).length ? set('submitErrors', fieldErrors) : unset('submitErrors'),
            );

            var _newState5 = _update3(state);

            return (0, _reactRecomponent.Update)(_newState5);
          }

        case 'START_SUBMIT':
          {
            var _newState6 = (0, _set2["default"])('submitting', true, state);

            return (0, _reactRecomponent.Update)(_newState6);
          }

        case 'TOUCH_ALL':
          {
            var _update4 = (0, _flow2["default"])((0, _set2["default"])('allTouched', true), (0, _set2["default"])('anyTouched', true));

            var _newState7 = _update4(state);

            return (0, _reactRecomponent.Update)(_newState7);
          }

        case 'VALIDATE':
          {
            var _newState8 = getStateAfterValidation(state);

            return (0, _reactRecomponent.Update)(_newState8);
          }

        default:
          return (0, _reactRecomponent.NoUpdate)();
      }
    }
  }]);
  return FormStateManager;
}(_reactRecomponent.ReComponent);

FormStateManager.defaultProps = {
  initialValues: {}
};
var _default = FormStateManager;
exports["default"] = _default;