"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = Image;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = _interopRequireDefault(require("react"));

var _styledComponents = _interopRequireDefault(require("styled-components"));

var _ViewBase = require("../../../internals/web/ViewBase");

var _utils = require("../utils");

var _ImageLoader = _interopRequireDefault(require("./ImageLoader"));

// @ts-ignore
var svgDataUriPattern = /^(data:image\/svg\+xml;utf8,)(.*)/;

var useImage = function useImage(uri) {
  var _React$useState = _react["default"].useState('LOADING'),
      _React$useState2 = (0, _slicedToArray2["default"])(_React$useState, 2),
      status = _React$useState2[0],
      setStatus = _React$useState2[1];

  var _React$useState3 = _react["default"].useState(undefined),
      _React$useState4 = (0, _slicedToArray2["default"])(_React$useState3, 2),
      imageMeta = _React$useState4[0],
      setImageMeta = _React$useState4[1];

  _react["default"].useEffect(function () {
    var cancelled = false;

    var requestId = _ImageLoader["default"].load(uri, function (img) {
      (0, _utils.getImageMeta)(img).then(function (meta) {
        if (!cancelled) {
          setStatus('LOADED');
          setImageMeta(meta);
        }
      }, function () {
        if (!cancelled) {
          // ignore error, treat it as having no metadata
          setStatus('LOADED');
          setImageMeta(undefined);
        }
      });
    }, function () {
      setStatus('ERROR');
    });

    return function () {
      cancelled = true;

      _ImageLoader["default"].abort(requestId);
    };
  }, [uri]);

  return {
    status: status,
    meta: imageMeta
  };
};

function Image(_ref) {
  var accessibilityLabel = _ref.accessibilityLabel,
      className = _ref.className,
      source = _ref.source,
      style = _ref.style,
      testID = _ref.testID;

  var _React$useState5 = _react["default"].useState(undefined),
      _React$useState6 = (0, _slicedToArray2["default"])(_React$useState5, 2),
      layout = _React$useState6[0],
      setLayout = _React$useState6[1];

  var handleLayout = function handleLayout(event) {
    setLayout(event.nativeEvent.layout);
  };

  var formattedUri = source.uri;
  var match = source.uri.match(svgDataUriPattern); // inline SVG markup may contain characters (e.g., #, ") that need to be escaped

  if (match) {
    var _match = (0, _slicedToArray2["default"])(match, 3),
        prefix = _match[1],
        svg = _match[2];

    var encodedSvg = encodeURIComponent(svg);
    formattedUri = "".concat(prefix).concat(encodedSvg);
  }

  var _useImage = useImage(formattedUri),
      status = _useImage.status,
      meta = _useImage.meta;

  return /*#__PURE__*/_react["default"].createElement(Container, {
    className: className,
    onLayout: handleLayout,
    testID: testID,
    style: style
  }, layout && /*#__PURE__*/_react["default"].createElement(TransformContainer, {
    $layout: layout,
    $orientation: meta && meta.Orientation
  }, status !== 'LOADING' && /*#__PURE__*/_react["default"].createElement(ImageContainer, {
    $status: status,
    $uri: formattedUri
  })), /*#__PURE__*/_react["default"].createElement(AccessibilityImg, {
    alt: accessibilityLabel,
    src: formattedUri
  }));
}

var Container = (0, _styledComponents["default"])(_ViewBase.DivViewBase).withConfig({
  displayName: "Imageweb__Container",
  componentId: "sc-1jdktuz-0"
})(["overflow:hidden;"]);

var TransformContainer = _styledComponents["default"].div.withConfig({
  displayName: "Imageweb__TransformContainer",
  componentId: "sc-1jdktuz-1"
})(["border-radius:inherit;max-height:100%;max-width:100%;position:relative;", ";", ";"], function (_ref2) {
  var $layout = _ref2.$layout,
      $orientation = _ref2.$orientation;
  return (0, _utils.isPortrait)($orientation) ? "\n    height: ".concat($layout.width, "px;\n    width: ").concat($layout.height, "px;\n  ") : "\n    height: ".concat($layout.height, "px;\n    width: ").concat($layout.width, "px;\n  ");
}, function (_ref3) {
  var $orientation = _ref3.$orientation;

  switch ($orientation) {
    case 2:
      return 'transform: scaleX(-1)';

    case 3:
      return 'transform: rotate(180deg)';

    case 4:
      return 'transform: scaleY(-1)';

    case 5:
      return 'transform: rotate(-90deg) scaleX(-1)';

    case 6:
      return 'transform: rotate(90deg)';

    case 7:
      return 'transform: rotate(90deg) scaleX(-1)';

    case 8:
      return 'transform: rotate(-90deg)';

    default:
      return '';
  }
});

var ImageContainer = _styledComponents["default"].div.withConfig({
  displayName: "Imageweb__ImageContainer",
  componentId: "sc-1jdktuz-2"
})(["position:absolute;top:0;bottom:0;left:0;right:0;background-position:center;background-repeat:no-repeat;background-size:contain;height:100%;width:100%;", ""], function (_ref4) {
  var $status = _ref4.$status,
      $uri = _ref4.$uri;
  return $status === 'ERROR' ? "\n    background: #eee;\n  " : "\n    animation-duration: 0.3s;\n    animation-iteration-count: 1;\n    animation-timing-function: ease-in;\n    background-image: url(".concat($uri, ");\n  ");
}); // invisible. Only used to allow right click and save


var AccessibilityImg = _styledComponents["default"].img.withConfig({
  displayName: "Imageweb__AccessibilityImg",
  componentId: "sc-1jdktuz-3"
})(["position:absolute;top:0;bottom:0;left:0;right:0;height:100%;opacity:0;width:100%;"]);